'use strict' // eslint-disable-line

const ExtractTextPlugin = require('extract-text-webpack-plugin')
const postcssFlexbugsFixes = require('postcss-flexbugs-fixes')
const autoprefixer = require('autoprefixer')
const pxtorem = require('postcss-pxtorem')
// get css webpack module rule
function cssloaders(notUseCssModules, extractTextPluginOptions) {
  const notProduction = process.env.NODE_ENV !== 'production'

  const shouldUseSourceMap = process.env.GENERATE_SOURCEMAP !== 'false' &&
    notProduction

  // whether use css-modules, determined by environment variable 'CSS_MODULES'
  const shouldUseCssModules = process.env.CSS_MODULES === 'true' && !notUseCssModules

  // whether use pxtorem plugin
  const shouldPxToRem = process.env.REACT_APP_PXTOREM !== 'undefined'

  // whether use preprocessor
  const preprocessor = process.env.PREPROCESSOR
  const shouldLess = preprocessor === 'less'
  const shouldSass = preprocessor === 'sass'

  const BROWSERS = [
    '>1%',
    'last 4 versions',
    'Firefox ESR',
    'not ie < 9', // 现代框架均不支持IE8及以下
    'iOS >= 8',
    'Android >= 4',
  ]

  const postcssPlugins = [
    postcssFlexbugsFixes,
    autoprefixer({
      browsers: BROWSERS,
      flexbox: 'no-2009',
    }),
  ]

  const preLoaders = [
    {
      loader: require.resolve('postcss-loader'),
      options: {
        ident: 'postcss',
        plugins: () => shouldPxToRem ? postcssPlugins.concat([
          pxtorem({
            rootValue: Number(process.env.REACT_APP_PXTOREM),
            unitPrecision: 5,
            propList: ['*'],
            selectorBlackList: [],
            replace: true,
            mediaQuery: false,
            minPixelValue: 0,
          }),
        ]) : postcssPlugins,
        sourceMap: shouldUseSourceMap,
      },
    },
  ]

  if(shouldLess) {
    preLoaders.push({
      loader: require.resolve('less-loader'),
      options: {
        sourceMap: shouldUseSourceMap,
      },
    })
  }

  if(shouldSass) {
    preLoaders.push({
      loader: require.resolve('sass-loader'),
      options: {
        sourceMap: shouldUseSourceMap,
      },
    })
  }

  // developt or test
  if (notProduction) {
    return [
      require.resolve('style-loader'),
      {
        loader: require.resolve('css-loader'),
        options: shouldUseCssModules ? {
          importLoaders: (shouldLess || shouldSass) ? 2 : 1,
          sourceMap: shouldUseSourceMap,
          modules: true,
          localIdentName: '[name]-[local]-[hash:base64:5]',
        } : {
          importLoaders: (shouldLess || shouldSass) ? 2 : 1,
          sourceMap: shouldUseSourceMap,
        },
      },
    ].concat(preLoaders)
  }

  // production
  return ExtractTextPlugin.extract(Object.assign(
    {
      fallback: {
        loader: require.resolve('style-loader'),
        options: {
          hmr: false,
        },
      },
      use: [{
        loader: require.resolve('css-loader'),
        options: shouldUseCssModules ? {
          importLoaders: (shouldLess || shouldSass) ? 2 : 1,
          minimize: true,
          sourceMap: shouldUseSourceMap,
          modules: true,
          localIdentName: '[name]-[local]-[hash:base64:5]',
        } : {
          importLoaders: (shouldLess || shouldSass) ? 2 : 1,
          minimize: true,
          sourceMap: shouldUseSourceMap,
        },
      }].concat(preLoaders),
    },
    extractTextPluginOptions
  ))
}

module.exports = cssloaders
